"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const opossum_1 = __importDefault(require("opossum"));
const enums_1 = require("./enums");
const utils_1 = require("./utils");
opossum_1.default = require("../opossum-state-fixed");
class Client {
    constructor(http) {
        this.globalOptions = {};
        this.http = http;
    }
    setGlobalOptions(options) {
        this.globalOptions = options;
    }
    request(name, url, options) {
        const circuit = Client.circuits.get(name) || this.createCircuit(name, options);
        return circuit.fire(url, options || {});
    }
    createCircuit(name, options) {
        const requestSender = (url, requestOptions) => {
            return new Promise((resolve, reject) => {
                this.http.request(url, requestOptions, (err, res) => {
                    if (err || !res) {
                        reject(err);
                        return;
                    }
                    const headers = requestOptions.headers || {};
                    const { flagHeaderNameToShowCurlOnResponse, responseHeaderNameForCurl } = this.globalOptions;
                    if (headers[flagHeaderNameToShowCurlOnResponse]) {
                        res.response.headers[responseHeaderNameForCurl] = utils_1.convertRequestToCurl(url, requestOptions);
                    }
                    if (options &&
                        options.json &&
                        res.body &&
                        res.response.headers["content-type"] &&
                        res.response.headers["content-type"].startsWith(enums_1.CONTENT_TYPE.ApplicationJson)) {
                        try {
                            res.json = JSON.parse(res.body);
                            resolve(res);
                        }
                        catch (error) {
                            reject(error);
                        }
                    }
                    else {
                        resolve(res);
                    }
                });
            });
        };
        const circuit = new opossum_1.default(requestSender, Object.assign({ name }, options));
        Client.circuits.set(name, circuit);
        return circuit;
    }
}
exports.Client = Client;
Client.circuits = new Map();
//# sourceMappingURL=client.js.map