"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zlib_1 = __importDefault(require("zlib"));
class Compression {
    static decompressStream(res, cb, handlerStream) {
        const buffer = [];
        const stream = handlerStream || res;
        stream.on('data', bufferChunk => {
            buffer.push(bufferChunk);
        });
        stream.on('error', err => {
            cb(err);
        });
        stream.on('end', (_) => {
            const fullBuffer = Buffer.concat(buffer);
            cb(null, fullBuffer.toString('utf8'));
        });
        if (handlerStream) {
            res.pipe(handlerStream);
        }
    }
    static compressBody(text, cb) {
        const buf = Buffer.from(text, 'utf8');
        zlib_1.default.gzip(buf, (err, data) => {
            cb(err, data);
        });
    }
    static handle(res, cb) {
        const encoding = res.headers["content-encoding"];
        if (encoding === 'gzip') {
            Compression.decompressStream(res, cb, zlib_1.default.createGunzip());
        }
        else if (encoding === 'br') {
            Compression.decompressStream(res, cb, zlib_1.default.createBrotliDecompress());
        }
        else {
            Compression.decompressStream(res, cb);
        }
    }
    static getSupportedStreams() {
        return 'gzip, deflate, br';
    }
}
exports.Compression = Compression;
//# sourceMappingURL=compression.js.map